@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: table.c,v 1.12 2002/12/18 20:02:41 nalin Exp $"
+#ident "$Id: table.c,v 1.13 2003/02/11 20:21:43 nalin Exp $"
 #include "../config.h"
 #include <sys/types.h>
 #include <assert.h>
@@ -404,7 +404,7 @@ _vte_table_extract_string(GValueArray **array,
 	ptr = g_malloc(sizeof(gunichar) * (arginfo->length + 1));
 	memcpy(ptr, arginfo->start, (arginfo->length * sizeof(gunichar)));
 	for (i = 0; i < arginfo->length; i++) {
-		ptr[i] &= ~(VTE_ISO2022_WIDTH_MASK);
+		ptr[i] &= ~(VTE_ISO2022_ENCODED_WIDTH_MASK);
 	}
 	ptr[arginfo->length] = '\0';
 	g_value_init(&value, G_TYPE_POINTER);
