@@ -16,7 +16,11 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#ident "$Id: vtefc.c,v 1.5 2003/04/21 02:50:46 nalin Exp $"
+
 #include "../config.h"
+
+#include <stdio.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include <pango/pango.h>
@@ -82,13 +86,17 @@ _vte_fc_transcribe_from_pango_font_description(FcPattern *pattern,
 
 	/* Set the font size for the pattern, or use a sensible default. */
 	if (pango_mask & PANGO_FONT_MASK_SIZE) {
-		size = (double) pango_font_description_get_size(font_desc);
-		size /= (double) PANGO_SCALE;
+		size = pango_font_description_get_size(font_desc);
+		size /= PANGO_SCALE;
 	}
 	FcPatternAddDouble(pattern, FC_SIZE, size);
 
 	/* Set the language for the pattern. */
+#if GTK_CHECK_VERSION(2,2,0)
 	context = gdk_pango_context_get_for_screen(gdk_screen_get_default());
+#else
+	context = gdk_pango_context_get();
+#endif
 	language = pango_context_get_language(context);
 	if (pango_language_to_string(language) != NULL) {
 		FcPatternAddString(pattern, FC_LANG,
@@ -114,14 +122,22 @@ static void
 _vte_fc_defaults_from_gtk(FcPattern *pattern)
 {
 	GtkSettings *settings;
+#if GTK_CHECK_VERSION(2,2,0)
+	GdkScreen *screen;
+#endif
 	GObjectClass *klass;
-	int i, antialias = -1, hinting = -1;
-	double d, dpi = -1;
+	int i, antialias = -1, hinting = -1, dpi = -1;
+	double d;
 	char *rgba = NULL, *hintstyle = NULL;
 	FcResult result;
 
 	/* Add any defaults configured for GTK+. */
-	settings = gtk_settings_get_for_screen(gdk_screen_get_default());
+#if GTK_CHECK_VERSION(2,2,0)
+	screen = gdk_screen_get_default();
+	settings = gtk_settings_get_for_screen(screen);
+#else
+	settings = gtk_settings_get_default();
+#endif
 	if (settings == NULL) {
 		return;
 	}
@@ -143,87 +159,84 @@ _vte_fc_defaults_from_gtk(FcPattern *pattern)
 
 	/* Pick up the antialiasing setting. */
 	if (antialias >= 0) {
-		result = FcPatternGetBool(pattern, FC_ANTIALIAS, 0, &i);
-		if (result == FcResultNoMatch) {
-			FcPatternAddBool(pattern, FC_ANTIALIAS, antialias > 0);
-		}
+		FcPatternDel(pattern, FC_ANTIALIAS);
+		FcPatternAddBool(pattern, FC_ANTIALIAS, antialias > 0);
 	}
 
 	/* Pick up the configured DPI setting. */
 	if (dpi >= 0) {
-		result = FcPatternGetDouble(pattern, FC_DPI, 0, &d);
-		if (result == FcResultNoMatch) {
-			FcPatternAddDouble(pattern, FC_DPI, dpi / 1024.0);
-		}
+		FcPatternDel(pattern, FC_DPI);
+		FcPatternAddDouble(pattern, FC_DPI, dpi / 1024.0);
 	}
 
 	/* Pick up the configured subpixel rendering setting. */
 	if (rgba != NULL) {
-		result = FcPatternGetInteger(pattern, FC_RGBA, 0, &i);
-		if (result == FcResultNoMatch) {
-			gboolean found;
+		gboolean found;
 
+		i = FC_RGBA_NONE;
+
+		if (strcmp(rgba, "none") == 0) {
 			i = FC_RGBA_NONE;
 			found = TRUE;
-
-			if (strcmp(rgba, "none") == 0) {
-				i = FC_RGBA_NONE;
-			} else
-			if (strcmp(rgba, "rgb") == 0) {
-				i = FC_RGBA_RGB;
-			} else
-			if (strcmp(rgba, "bgr") == 0) {
-				i = FC_RGBA_BGR;
-			} else
-			if (strcmp(rgba, "vrgb") == 0) {
-				i = FC_RGBA_VRGB;
-			} else
-			if (strcmp(rgba, "vbgr") == 0) {
-				i = FC_RGBA_VBGR;
-			} else {
-				found = FALSE;
-			}
-			if (found) {
-				FcPatternAddInteger(pattern, FC_RGBA, i);
-			}
+		} else
+		if (strcmp(rgba, "rgb") == 0) {
+			i = FC_RGBA_RGB;
+			found = TRUE;
+		} else
+		if (strcmp(rgba, "bgr") == 0) {
+			i = FC_RGBA_BGR;
+			found = TRUE;
+		} else
+		if (strcmp(rgba, "vrgb") == 0) {
+			i = FC_RGBA_VRGB;
+			found = TRUE;
+		} else
+		if (strcmp(rgba, "vbgr") == 0) {
+			i = FC_RGBA_VBGR;
+			found = TRUE;
+		} else {
+			found = FALSE;
+		}
+		if (found) {
+			FcPatternDel(pattern, FC_RGBA);
+			FcPatternAddInteger(pattern, FC_RGBA, i);
 		}
 	}
 
 	/* Pick up the configured hinting setting. */
 	if (hinting >= 0) {
-		result = FcPatternGetBool(pattern, FC_HINTING, 0, &i);
-		if (result == FcResultNoMatch) {
-			FcPatternAddBool(pattern, FC_HINTING, hinting > 0);
-		}
+		FcPatternDel(pattern, FC_HINTING);
+		FcPatternAddBool(pattern, FC_HINTING, hinting > 0);
 	}
 
 #ifdef FC_HINT_STYLE
 	/* Pick up the default hinting style. */
 	if (hintstyle != NULL) {
-		result = FcPatternGetInteger(pattern, FC_HINT_STYLE, 0, &i);
-		if (result == FcResultNoMatch) {
-			gboolean found;
+		gboolean found;
 
+		i = FC_HINT_NONE;
+
+		if (strcmp(hintstyle, "hintnone") == 0) {
 			i = FC_HINT_NONE;
 			found = TRUE;
-
-			if (strcmp(hintstyle, "hintnone") == 0) {
-				i = FC_HINT_NONE;
-			} else
-			if (strcmp(hintstyle, "hintslight") == 0) {
-				i = FC_HINT_SLIGHT;
-			} else
-			if (strcmp(hintstyle, "hintmedium") == 0) {
-				i = FC_HINT_MEDIUM;
-			} else
-			if (strcmp(hintstyle, "hintfull") == 0) {
-				i = FC_HINT_FULL;
-			} else {
-				found = FALSE;
-			}
-			if (found) {
-				FcPatternAddInteger(pattern, FC_HINT_STYLE, i);
-			}
+		} else
+		if (strcmp(hintstyle, "hintslight") == 0) {
+			i = FC_HINT_SLIGHT;
+			found = TRUE;
+		} else
+		if (strcmp(hintstyle, "hintmedium") == 0) {
+			i = FC_HINT_MEDIUM;
+			found = TRUE;
+		} else
+		if (strcmp(hintstyle, "hintfull") == 0) {
+			i = FC_HINT_FULL;
+			found = TRUE;
+		} else {
+			found = FALSE;
+		}
+		if (found) {
+			FcPatternDel(pattern, FC_HINT_STYLE);
+			FcPatternAddInteger(pattern, FC_HINT_STYLE, i);
 		}
 	}
 #endif
@@ -266,6 +279,7 @@ _vte_fc_patterns_from_pango_font_desc(const PangoFontDescription *font_desc,
 			tmp = FcFontRenderPrepare(NULL,
 						  pattern,
 						  fontset->fonts[i]);
+			_vte_fc_defaults_from_gtk(tmp);
 			g_array_append_val(pattern_array, tmp);
 		}
 		FcFontSetDestroy(fontset);
@@ -277,6 +291,7 @@ _vte_fc_patterns_from_pango_font_desc(const PangoFontDescription *font_desc,
 		match = FcFontMatch(NULL, pattern, &result);
 		if (result == FcResultMatch) {
 			tmp = FcPatternDuplicate(match);
+			_vte_fc_defaults_from_gtk(tmp);
 			g_array_append_val(pattern_array, tmp);
 			ret = TRUE;
 		} else {
