@@ -16,6 +16,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#ident "$Id: vtexft.c,v 1.8 2003/04/24 18:17:17 nalin Exp $"
+
 #include "../config.h"
 
 #ifndef X_DISPLAY_MISSING
@@ -33,6 +35,14 @@
 #include "vtefc.h"
 #include "vtexft.h"
 
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#define _(String) dgettext(PACKAGE, String)
+#else
+#define _(String) String
+#define bindtextdomain(package,dir)
+#endif
+ 
 #define FONT_INDEX_FUDGE 10
 #define CHAR_WIDTH_FUDGE 10
 
@@ -230,9 +240,8 @@ _vte_xft_font_for_char(struct _vte_xft_font *font, gunichar c)
 }
 
 static int
-_vte_xft_char_width(struct _vte_xft_font *font, gunichar c)
+_vte_xft_char_width(struct _vte_xft_font *font, XftFont *ftfont, gunichar c)
 {
-	XftFont *ftfont;
 	XGlyphInfo extents;
 	gpointer p = GINT_TO_POINTER(c);
 	int i;
@@ -257,7 +266,6 @@ _vte_xft_char_width(struct _vte_xft_font *font, gunichar c)
 	}
 
 	/* Compute and store the width. */
-	ftfont = _vte_xft_font_for_char(font, c);
 	memset(&extents, 0, sizeof(extents));
 	if (ftfont != NULL) {
 		XftTextExtents32(GDK_DISPLAY(), ftfont, &c, 1, &extents);
@@ -592,6 +600,36 @@ _vte_xft_get_using_fontconfig(struct _vte_draw *draw)
 	return TRUE;
 }
 
+#if 0
+/* We're just a barely-there wrapper around XftDrawCharFontSpec(). */
+static void
+_vte_xft_drawcharfontspec(XftDraw *draw, XftColor *color,
+			  XftCharFontSpec *specs, int n)
+{
+	XftDrawCharFontSpec(draw, color, specs, n);
+}
+#else
+/* We need to break down the draw request into runs which use the same
+ * font, to work around a bug which appears to be in Xft and which I
+ * haven't pinned down yet. */
+static void
+_vte_xft_drawcharfontspec(XftDraw *draw, XftColor *color,
+			  XftCharFontSpec *specs, int n)
+{
+	int i, j;
+	i = j = 0;
+	while (i < n) {
+		for (j = i + 1; j < n; j++) {
+			if (specs[i].font != specs[j].font) {
+				break;
+			}
+		}
+		XftDrawCharFontSpec(draw, color, specs + i, j - i);
+		i = j;
+	}
+}
+#endif
+
 static void
 _vte_xft_draw_text(struct _vte_draw *draw,
 		   struct _vte_draw_text_request *requests, gsize n_requests,
@@ -615,7 +653,9 @@ _vte_xft_draw_text(struct _vte_draw *draw,
 						       requests[i].c);
 		if (specs[j].font != NULL) {
 			specs[j].x = requests[i].x - data->x_offs;
-			width = _vte_xft_char_width(data->font, requests[i].c);
+			width = _vte_xft_char_width(data->font,
+						    specs[j].font,
+						    requests[i].c);
 			if (width != 0) {
 				pad = requests[i].columns * draw->width - width;
 				pad = CLAMP(pad / 2, 0, draw->width);
@@ -625,7 +665,7 @@ _vte_xft_draw_text(struct _vte_draw *draw,
 			specs[j].ucs4 = requests[i].c;
 			j++;
 		} else {
-			g_warning("Cannot draw character U%04x.\n",
+			g_warning(_("Can not draw character U+%04x.\n"),
 				  requests[i].c);
 		}
 	}
@@ -637,7 +677,8 @@ _vte_xft_draw_text(struct _vte_draw *draw,
 			       0xffff : (alpha << 8);
 		if (XftColorAllocValue(data->display, data->visual,
 				       data->colormap, &rcolor, &ftcolor)) {
-			XftDrawCharFontSpec(data->draw, &ftcolor, specs, j);
+			_vte_xft_drawcharfontspec(data->draw, &ftcolor,
+						  specs, j);
 			XftColorFree(data->display, data->visual,
 				     data->colormap, &ftcolor);
 		}
